from .api_base import APIBase
import requests


class SoundCloudCharts(APIBase):

    def __init__(self, proxies=None):
        """
        Initialize the class. Calls the init method of APIBase and passes through root URL and proxy info.

        :param proxies: Dictionary of proxy information, if needed
        """
        super().__init__(root='https://api-v2.soundcloud.com/', proxies=proxies)
        self.client_id = self.get_client_id()  # Client ID needed to make API calls

    def get_client_id(self):
        """
        Gets the client ID for the API.
        The ID is located in a Javascript file that was found on the chart page source.
        The ID periodically will expire, so this function will look for the most recent ID upon initialization

        :return: Client ID as a string
        """
        # The location of the client ID
        js_url = 'https://a-v2.sndcdn.com/assets/app-5c2fc88-9c19ae7-3.js'

        # The file is large, so stream it and take just the first 10,000 characters
        # The ID is located within these 10,000 characters, but this may need to change in the future
        chunk_size = 10000
        with requests.get(js_url, stream=True, proxies=self.proxies) as r:
            raw_text = next(r.iter_content(chunk_size=chunk_size)).decode('utf-8')

        pattern = 'client_id:"'
        len_of_id = 32  # The client ID is currently 32 characters long
        start = raw_text.find(pattern) + len(pattern)
        end = start + len_of_id

        return raw_text[start:end]

    def get_chart(self, kind='top', genre='all-music', region=None, limit=50, offset=0):
        """
        Get a Soundcloud chart

        :param kind: Chart type. Either "top" or "trending". "trending" is New & Hot
        :param genre: Song genre
        :param region: Country, or None for global
        :param limit: The maximum amount of songs to return
        :param offset: The index of the first item returned

        :return: JSON data of songs on chart
        """
        url = self.root + 'charts'
        if 'soundcloud:genres:' not in genre:
            genre = 'soundcloud:genres:' + genre

        if region is None:
            return self._get(url, client_id=self.client_id, kind=kind, genre=genre, limit=limit, offset=offset)
        else:
            if 'soundcloud:regions:' not in region:
                region = 'soundcloud:regions:' + region
            return self._get(url, client_id=self.client_id, kind=kind, genre=genre, region=region,
                             limit=limit, offset=offset)




import requests
from requests.exceptions import HTTPError
import time


class APIBase:
    """
    This class is to be used as a base to build an API library.
    Authorization token generation and endpoint functions must be written
    """

    def __init__(self, root, proxies=None, requests_session=True, max_retries=10, requests_timeout=None):
        """
        Initialize the class

        :param: root: Root URL for the API
        :param proxies: A dictionary of proxies, if needed
        :param requests_session: Use request Sessions class. Speeds up API calls significantly when set to True
        :param max_retries: Maximum amount of times to retry an API call before stopping
        :param requests_timeout: Number of seconds requests should wait before timing out
        """

        self.proxies = proxies
        self.token_str = ""  # Encrypted API token. This will need to be set manually or by a method of a subclass
        self.root = root
        self.max_retries = max_retries
        self.requests_timeout = requests_timeout
        if requests_session:
            self._session = requests.Session()
        else:
            self._session = requests.api  # individual calls, slower

    def _auth_headers(self):
        """
        Get header for API request

        :return: header in dictionary format
        """
        if self.token_str:
            return {'Authorization': 'Bearer {}'.format(self.token_str)}
        else:
            return {}

    def _call(self, method, url, params):
        """
        Make a call to the API

        :param method: 'GET', 'POST', 'DELETE', or 'PUT'
        :param url: URL of API endpoint
        :param params: API paramaters

        :return: JSON data from the API
        """
        if not url.startswith('http'):
            url = self.root + url
        headers = self._auth_headers()
        headers['Content-Type'] = 'application/json'

        r = self._session.request(method, url,
                                  headers=headers,
                                  proxies=self.proxies,
                                  params=params,
                                  timeout=self.requests_timeout)
        r.raise_for_status()  # Check for error
        return r.json()

    def _get(self, url, **kwargs):
        """
        GET request from the API

        :param url: URL for API endpoint

        :return: JSON data from the API
        """
        retries = self.max_retries
        delay = 1
        while retries > 0:
            try:
                return self._call('GET', url, kwargs)
            except HTTPError as e:  # Retry for some known issues
                retries -= 1
                status = e.response.status_code
                if status == 429 or (500 <= status < 600):
                    if retries < 0:
                        raise
                    else:
                        sleep_seconds = int(e.headers.get('Retry-After', delay))
                        print('retrying ...' + str(sleep_seconds) + ' secs')
                        time.sleep(sleep_seconds + 1)
                        delay += 1
                else:
                    raise
            except Exception as e:
                print('exception', str(e))
                retries -= 1
                if retries >= 0:
                    print('retrying ...' + str(delay) + 'secs')
                    time.sleep(delay + 1)
                    delay += 1
                else:
                    raise

    def _post(self, url, **kwargs):
        """
        POST request from the API

        :param url: URL for API endpoint

        :return: JSON data from the API
        """
        return self._call('POST', url, kwargs)

    def _delete(self, url, **kwargs):
        """
        DELETE request from the API

        :param url: URL for API endpoint

        :return: JSON data from the API
        """
        return self._call('DELETE', url, kwargs)

    def _put(self, url, **kwargs):
        """
        PUT request from the API

        :param url: URL for API endpoint

        :return: JSON data from the API
        """
        return self._call('PUT', url, kwargs)


from setuptools import setup

with open('README.md', 'r') as fh:
    long_description = fh.read()

setup(
    name='soundcloud-charts-api',
    url='https://github.com/mpalazzolo/soundcloud-charts-api',
    version='1.0.2',
    packages=['soundcloudcharts'],
    license='LICENSE.txt',
    author='Matt Palazzolo',
    author_email='mattpalazzolo@gmail.com',
    description='A python wrapper for the Soundcloud Chart API',
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=[
        'requests>=2.22.0'
    ],
)